@@ -20,7 +20,7 @@
 #       MA 02110-1301, USA.
 
 import os, sys, ConfigParser, gtk, pango, locale, subprocess, base64, webbrowser
-import cons, support, pgsc_spellcheck
+import cons, support, codeboxes, pgsc_spellcheck
 if cons.HAS_APPINDICATOR: import appindicator
 
 ICONS_SIZE = {1: gtk.ICON_SIZE_MENU, 2: gtk.ICON_SIZE_SMALL_TOOLBAR, 3: gtk.ICON_SIZE_LARGE_TOOLBAR,
@@ -37,6 +37,21 @@ SPECIAL_CHARS_DEFAULT = "“”„•☐☑☒…‰€©®™°↓↑→←↔
 
 SPELL_CHECK_LANG_DEFAULT = locale.getdefaultlocale()[0]
 
+def get_node_path_from_str(str_path_list_of_str):
+    str_path_list_of_str = str(str_path_list_of_str)
+    if str_path_list_of_str.startswith(cons.CHAR_PARENTH_OPEN)\
+    and str_path_list_of_str.endswith(cons.CHAR_PARENTH_CLOSE):
+        str_path_list_of_str = str_path_list_of_str[1:-1].replace(cons.CHAR_COMMA, "")
+    path_list_of_str = str_path_list_of_str.split()
+    path_list_of_int = [int(element) for element in path_list_of_str]
+    return tuple(path_list_of_int)
+
+def get_node_path_str_from_path(tree_path):
+    path_list_of_str = []
+    for element in tree_path:
+        path_list_of_str.append( str(element) )
+    return " ".join(path_list_of_str)
+
 def config_file_load(inst):
     """Load the Preferences from Config File"""
     if os.path.isfile(cons.CONFIG_PATH):
@@ -59,11 +74,8 @@ def config_file_load(inst):
             inst.window.resize(win_size[0], win_size[1])
         inst.hpaned_pos = config.getint(section, "hpaned_pos") if config.has_option(section, "hpaned_pos") else 170
         if config.has_option(section, "node_path"):
-            # restor the selected node
-            str_path_list_of_str = config.get(section, "node_path")
-            path_list_of_str = str_path_list_of_str.split()
-            path_list_of_int = [int(element) for element in path_list_of_str]
-            inst.node_path = tuple(path_list_of_int)
+            # restore the selected node
+            inst.node_path = get_node_path_from_str(config.get(section, "node_path"))
             inst.cursor_position = config.getint(section, "cursor_position") if config.has_option(section, "cursor_position") else 0
         else: inst.node_path = None
         inst.recent_docs = []
@@ -84,12 +96,25 @@ def config_file_load(inst):
         section = "tree"
         inst.rest_exp_coll = config.getint(section, "rest_exp_coll") if config.has_option(section, "rest_exp_coll") else 0
         inst.expanded_collapsed_string = config.get(section, "expanded_collapsed_string") if config.has_option(section, "expanded_collapsed_string") else ""
+        inst.expcollnam1 = unicode(config.get(section, "expcollnam1"), cons.STR_UTF8, cons.STR_IGNORE) if config.has_option(section, "expcollnam1") else ""
+        inst.expcollstr1 = config.get(section, "expcollstr1") if config.has_option(section, "expcollstr1") else ""
+        inst.expcollsel1 = config.get(section, "expcollsel1") if config.has_option(section, "expcollsel1") else ""
+        inst.expcollcur1 = config.getint(section, "expcollcur1") if config.has_option(section, "expcollcur1") else 0
+        inst.expcollnam2 = unicode(config.get(section, "expcollnam2"), cons.STR_UTF8, cons.STR_IGNORE) if config.has_option(section, "expcollnam2") else ""
+        inst.expcollstr2 = config.get(section, "expcollstr2") if config.has_option(section, "expcollstr2") else ""
+        inst.expcollsel2 = config.get(section, "expcollsel2") if config.has_option(section, "expcollsel2") else ""
+        inst.expcollcur2 = config.getint(section, "expcollcur2") if config.has_option(section, "expcollcur2") else 0
+        inst.expcollnam3 = unicode(config.get(section, "expcollnam3"), cons.STR_UTF8, cons.STR_IGNORE) if config.has_option(section, "expcollnam3") else ""
+        inst.expcollstr3 = config.get(section, "expcollstr3") if config.has_option(section, "expcollstr3") else ""
+        inst.expcollsel3 = config.get(section, "expcollsel3") if config.has_option(section, "expcollsel3") else ""
+        inst.expcollcur3 = config.getint(section, "expcollcur3") if config.has_option(section, "expcollcur3") else 0
         inst.nodes_icons = config.get(section, "nodes_icons") if config.has_option(section, "nodes_icons") else "c"
         inst.tree_right_side = config.getboolean(section, "tree_right_side") if config.has_option(section, "tree_right_side") else False
         inst.cherry_wrap_width = config.getint(section, "cherry_wrap_width") if config.has_option(section, "cherry_wrap_width") else 130
 
         section = "editor"
-        inst.syntax_highlighting = config.get(section, "syntax_highlighting") if config.has_option(section, "syntax_highlighting") else cons.CUSTOM_COLORS_ID
+        inst.syntax_highlighting = config.get(section, "syntax_highlighting") if config.has_option(section, "syntax_highlighting") else cons.RICH_TEXT_ID
+        inst.auto_syn_highl = config.get(section, "auto_syn_highl") if config.has_option(section, "auto_syn_highl") else "sh"
         inst.style_scheme = config.get(section, "style_scheme") if config.has_option(section, "style_scheme") else cons.STYLE_SCHEME_DEFAULT
         inst.enable_spell_check = config.getboolean(section, "enable_spell_check") if config.has_option(section, "enable_spell_check") else False
         inst.spell_check_lang = config.get(section, "spell_check_lang") if config.has_option(section, "spell_check_lang") else SPELL_CHECK_LANG_DEFAULT
@@ -97,9 +122,11 @@ def config_file_load(inst):
         inst.spaces_instead_tabs = config.getboolean(section, "spaces_instead_tabs") if config.has_option(section, "spaces_instead_tabs") else True
         inst.tabs_width = config.getint(section, "tabs_width") if config.has_option(section, "tabs_width") else 4
         inst.anchor_size = config.getint(section, "anchor_size") if config.has_option(section, "anchor_size") else 16
+        inst.embfile_size = config.getint(section, "embfile_size") if config.has_option(section, "embfile_size") else 48
         inst.line_wrapping = config.getboolean(section, "line_wrapping") if config.has_option(section, "line_wrapping") else True
         inst.wrapping_indent = config.getint(section, "wrapping_indent") if config.has_option(section, "wrapping_indent") else -14
         inst.auto_indent = config.getboolean(section, "auto_indent") if config.has_option(section, "auto_indent") else True
+        inst.rt_show_white_spaces = config.getboolean(section, "rt_show_white_spaces") if config.has_option(section, "rt_show_white_spaces") else False
         inst.show_white_spaces = config.getboolean(section, "show_white_spaces") if config.has_option(section, "show_white_spaces") else True
         inst.highl_curr_line = config.getboolean(section, "highl_curr_line") if config.has_option(section, "highl_curr_line") else True
         inst.h_rule = config.get(section, "h_rule") if config.has_option(section, "h_rule") else HORIZONTAL_RULE
@@ -163,14 +190,17 @@ def config_file_load(inst):
         inst.reload_doc_last = config.getboolean(section, "reload_doc_last") if config.has_option(section, "reload_doc_last") else True
         inst.enable_mod_time_sentinel = config.getboolean(section, "mod_time_sent") if config.has_option(section, "mod_time_sent") else True
         inst.backup_copy = config.getboolean(section, "backup_copy") if config.has_option(section, "backup_copy") else True
+        inst.backup_num = config.getint(section, "backup_num") if config.has_option(section, "backup_num") else 3
         inst.autosave_on_quit = config.getboolean(section, "autosave_on_quit") if config.has_option(section, "autosave_on_quit") else False
         inst.limit_undoable_steps = config.getint(section, "limit_undoable_steps") if config.has_option(section, "limit_undoable_steps") else 20
+        #print "read", cons.CONFIG_PATH, "('%s', '%s')" % (inst.file_name, inst.file_dir)
     else:
         inst.file_dir = ""
         inst.file_name = ""
         inst.node_path = None
         inst.curr_colors = {'f':None, 'b':None}
-        inst.syntax_highlighting = cons.CUSTOM_COLORS_ID
+        inst.syntax_highlighting = cons.RICH_TEXT_ID
+        inst.auto_syn_highl = "sh"
         inst.style_scheme = cons.STYLE_SCHEME_DEFAULT
         inst.tree_font = "Sans 8" # default tree font
         inst.text_font = "Sans 9" # default text font
@@ -192,6 +222,7 @@ def config_file_load(inst):
         inst.spaces_instead_tabs = True
         inst.tabs_width = 4
         inst.anchor_size = 16
+        inst.embfile_size = 48
         inst.line_wrapping = True
         inst.wrapping_indent = -14
         inst.auto_indent = True
@@ -201,6 +232,9 @@ def config_file_load(inst):
         inst.win_is_maximized = False
         inst.rest_exp_coll = 0
         inst.expanded_collapsed_string = ""
+        inst.expcollnam1 = ""
+        inst.expcollnam2 = ""
+        inst.expcollnam3 = ""
         inst.pick_dir = ""
         inst.link_type = cons.LINK_TYPE_WEBS
         inst.toolbar_icon_size = 1
@@ -224,8 +258,10 @@ def config_file_load(inst):
         inst.reload_doc_last = True
         inst.enable_mod_time_sentinel = True
         inst.backup_copy = True
+        inst.backup_num = 3
         inst.autosave_on_quit = False
         inst.tree_right_side = False
+        inst.rt_show_white_spaces = False
         inst.show_white_spaces = True
         inst.highl_curr_line = True
         inst.hpaned_pos = 170
@@ -233,6 +269,7 @@ def config_file_load(inst):
         inst.nodes_icons = "c"
         inst.recent_docs = []
         inst.toolbar_visible = True
+        print "missing", cons.CONFIG_PATH
 
 def config_file_apply(inst):
     """Apply the Preferences from Config File"""
@@ -277,10 +314,7 @@ def config_file_save(inst):
         config.set(section, "win_size_h", win_size[1])
     config.set(section, "hpaned_pos", inst.hpaned.get_property('position'))
     if inst.curr_tree_iter:
-        path_list_of_str = []
-        for element in inst.treestore.get_path(inst.curr_tree_iter):
-            path_list_of_str.append( str(element) )
-        config.set(section, "node_path", " ".join(path_list_of_str))
+        config.set(section, "node_path", get_node_path_str_from_path(inst.treestore.get_path(inst.curr_tree_iter)))
         config.set(section, "cursor_position", inst.curr_buffer.get_property(cons.STR_CURSOR_POSITION))
     if inst.recent_docs:
         temp_recent_docs = []
@@ -303,6 +337,21 @@ def config_file_save(inst):
     if inst.rest_exp_coll == 0:
         get_tree_expanded_collapsed_string(inst)
         config.set(section, "expanded_collapsed_string", inst.expanded_collapsed_string)
+    if inst.expcollnam1 and inst.expcollnam1 != inst.file_name:
+        config.set(section, "expcollnam1", inst.expcollnam1)
+        config.set(section, "expcollstr1", inst.expcollstr1)
+        config.set(section, "expcollsel1", inst.expcollsel1)
+        config.set(section, "expcollcur1", inst.expcollcur1)
+    if inst.expcollnam2 and inst.expcollnam2 != inst.file_name:
+        config.set(section, "expcollnam2", inst.expcollnam2)
+        config.set(section, "expcollstr2", inst.expcollstr2)
+        config.set(section, "expcollsel2", inst.expcollsel2)
+        config.set(section, "expcollcur2", inst.expcollcur2)
+    if inst.expcollnam3 and inst.expcollnam3 != inst.file_name:
+        config.set(section, "expcollnam3", inst.expcollnam3)
+        config.set(section, "expcollstr3", inst.expcollstr3)
+        config.set(section, "expcollsel3", inst.expcollsel3)
+        config.set(section, "expcollcur3", inst.expcollcur3)
     config.set(section, "nodes_icons", inst.nodes_icons)
     config.set(section, "tree_right_side", inst.tree_right_side)
     config.set(section, "cherry_wrap_width", inst.cherry_wrap_width)
@@ -310,6 +359,7 @@ def config_file_save(inst):
     section = "editor"
     config.add_section(section)
     config.set(section, "syntax_highlighting", inst.syntax_highlighting)
+    config.set(section, "auto_syn_highl", inst.auto_syn_highl)
     config.set(section, "style_scheme", inst.style_scheme)
     config.set(section, "spell_check_lang", inst.spell_check_lang)
     config.set(section, "enable_spell_check", inst.enable_spell_check)
@@ -317,9 +367,11 @@ def config_file_save(inst):
     config.set(section, "spaces_instead_tabs", inst.spaces_instead_tabs)
     config.set(section, "tabs_width", inst.tabs_width)
     config.set(section, "anchor_size", inst.anchor_size)
+    config.set(section, "embfile_size", inst.embfile_size)
     config.set(section, "line_wrapping", inst.line_wrapping)
     config.set(section, "wrapping_indent", inst.wrapping_indent)
     config.set(section, "auto_indent", inst.auto_indent)
+    config.set(section, "rt_show_white_spaces", inst.rt_show_white_spaces)
     config.set(section, "show_white_spaces", inst.show_white_spaces)
     config.set(section, "highl_curr_line", inst.highl_curr_line)
     config.set(section, "h_rule", inst.h_rule)
@@ -372,11 +424,13 @@ def config_file_save(inst):
     config.set(section, "reload_doc_last", inst.reload_doc_last)
     config.set(section, "mod_time_sent", inst.enable_mod_time_sentinel)
     config.set(section, "backup_copy", inst.backup_copy)
+    config.set(section, "backup_num", inst.backup_num)
     config.set(section, "autosave_on_quit", inst.autosave_on_quit)
     config.set(section, "limit_undoable_steps", inst.limit_undoable_steps)
 
     with open(cons.CONFIG_PATH, 'wb') as configfile:
         config.write(configfile)
+        #print "saved", cons.CONFIG_PATH, "('%s', '%s')" % (inst.file_name, inst.file_dir)
 
 def get_tree_expanded_collapsed_string(inst):
     """Returns a String Containing the Info about Expanded and Collapsed Nodes"""
@@ -401,7 +455,7 @@ def get_tree_expanded_collapsed_string_iter(tree_iter, inst):
 def set_tree_expanded_collapsed_string(inst):
     """Parses the String Containing the Info about Expanded and Collapsed Nodes"""
     inst.treeview.collapse_all()
-    if inst.expanded_collapsed_string == "": return
+    if not inst.expanded_collapsed_string: return
     expanded_collapsed_dict = {}
     expanded_collapsed_vector = inst.expanded_collapsed_string.split('_')
     for element in expanded_collapsed_vector:
@@ -422,7 +476,24 @@ def set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict,
         set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, inst)
         tree_iter = inst.treestore.iter_next(tree_iter)
 
-def preferences_tab_all_nodes(dad, vbox_all_nodes):
+def set_tree_path_and_cursor_pos(inst):
+    """Try to set node path and cursor pos"""
+    if inst.node_path:
+        try: node_iter_to_focus = inst.treestore.get_iter(inst.node_path)
+        except: node_iter_to_focus = None
+        if node_iter_to_focus:
+            inst.treeview_safe_set_cursor(node_iter_to_focus)
+            inst.sourceview.grab_focus()
+            inst.curr_buffer.place_cursor(inst.curr_buffer.get_iter_at_offset(inst.cursor_position))
+            inst.sourceview.scroll_to_mark(inst.curr_buffer.get_insert(), 0.3)
+    else: node_iter_to_focus = None
+    if not node_iter_to_focus:
+        node_iter_to_focus = inst.treestore.get_iter_first()
+        if node_iter_to_focus:
+            inst.treeview.set_cursor(inst.treestore.get_path(node_iter_to_focus))
+            inst.sourceview.grab_focus()
+
+def preferences_tab_all_nodes(dad, vbox_all_nodes, pref_dialog):
     """Preferences Dialog, All Nodes Tab"""
     for child in vbox_all_nodes.get_children(): child.destroy()
 
@@ -559,7 +630,7 @@ def preferences_tab_all_nodes(dad, vbox_all_nodes):
         dad.h_rule = entry.get_text()
     entry_horizontal_rule.connect('changed', on_entry_horizontal_rule_changed)
 
-def preferences_tab_text_nodes(dad, vbox_text_nodes):
+def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
     """Preferences Dialog, Text Nodes Tab"""
     for child in vbox_text_nodes.get_children(): child.destroy()
 
@@ -629,6 +700,16 @@ def preferences_tab_text_nodes(dad, vbox_text_nodes):
 
     hbox_misc_text = gtk.HBox()
     hbox_misc_text.set_spacing(4)
+    checkbutton_rt_show_white_spaces = gtk.CheckButton(_("Show White Spaces"))
+    checkbutton_rt_show_white_spaces.set_active(dad.rt_show_white_spaces)
+    hbox_embfile_size = gtk.HBox()
+    hbox_embfile_size.set_spacing(4)
+    label_embfile_size = gtk.Label(_("Embedded File Icon Size"))
+    adj_embfile_size = gtk.Adjustment(value=dad.embfile_size, lower=1, upper=1000, step_incr=1)
+    spinbutton_embfile_size = gtk.SpinButton(adj_embfile_size)
+    spinbutton_embfile_size.set_value(dad.embfile_size)
+    hbox_embfile_size.pack_start(label_embfile_size, expand=False)
+    hbox_embfile_size.pack_start(spinbutton_embfile_size, expand=False)
     label_limit_undoable_steps = gtk.Label(_("Limit of Undoable Steps Per Node"))
     adj_limit_undoable_steps = gtk.Adjustment(value=dad.limit_undoable_steps, lower=1, upper=10000, step_incr=1)
     spinbutton_limit_undoable_steps = gtk.SpinButton(adj_limit_undoable_steps)
@@ -637,6 +718,8 @@ def preferences_tab_text_nodes(dad, vbox_text_nodes):
     hbox_misc_text.pack_start(spinbutton_limit_undoable_steps, expand=False)
 
     vbox_misc_text = gtk.VBox()
+    vbox_misc_text.pack_start(checkbutton_rt_show_white_spaces, expand=False)
+    vbox_misc_text.pack_start(hbox_embfile_size, expand=False)
     vbox_misc_text.pack_start(hbox_misc_text, expand=False)
     frame_misc_text = gtk.Frame(label="<b>"+_("Miscellaneous")+"</b>")
     frame_misc_text.get_label_widget().set_use_markup(True)
@@ -654,7 +737,7 @@ def preferences_tab_text_nodes(dad, vbox_text_nodes):
         if dad.enable_spell_check:
             dad.spell_check_set_on()
             set_checkbutton_spell_check_model()
-        else: dad.spell_check_set_off()
+        else: dad.spell_check_set_off(True)
         combobox_spell_check_lang.set_sensitive(dad.enable_spell_check)
     checkbutton_spell_check.connect('toggled', on_checkbutton_spell_check_toggled)
     def on_combobox_spell_check_lang_changed(combobox):
@@ -664,12 +747,12 @@ def preferences_tab_text_nodes(dad, vbox_text_nodes):
     combobox_spell_check_lang.connect('changed', on_combobox_spell_check_lang_changed)
     def on_colorbutton_text_fg_color_set(colorbutton):
         dad.rt_def_fg = "#" + dad.html_handler.rgb_to_24(colorbutton.get_color().to_string()[1:])
-        if dad.curr_tree_iter and dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if dad.curr_tree_iter and dad.syntax_highlighting == cons.RICH_TEXT_ID:
             dad.sourceview.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse(dad.rt_def_fg))
     colorbutton_text_fg.connect('color-set', on_colorbutton_text_fg_color_set)
     def on_colorbutton_text_bg_color_set(colorbutton):
         dad.rt_def_bg = "#" + dad.html_handler.rgb_to_24(colorbutton.get_color().to_string()[1:])
-        if dad.curr_tree_iter and dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if dad.curr_tree_iter and dad.syntax_highlighting == cons.RICH_TEXT_ID:
             dad.sourceview.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse(dad.rt_def_bg))
     colorbutton_text_bg.connect('color-set', on_colorbutton_text_bg_color_set)
     def on_radiobutton_rt_col_light_toggled(radiobutton):
@@ -695,6 +778,17 @@ def preferences_tab_text_nodes(dad, vbox_text_nodes):
         colorbutton_text_fg.set_sensitive(True)
         colorbutton_text_bg.set_sensitive(True)
     radiobutton_rt_col_custom.connect('toggled', on_radiobutton_rt_col_custom_toggled)
+    def on_checkbutton_rt_show_white_spaces_toggled(checkbutton):
+        dad.rt_show_white_spaces = checkbutton.get_active()
+        if dad.syntax_highlighting == cons.RICH_TEXT_ID:
+            dad.sourceview.set_draw_spaces(codeboxes.DRAW_SPACES_FLAGS if dad.rt_show_white_spaces else 0)
+    checkbutton_rt_show_white_spaces.connect('toggled', on_checkbutton_rt_show_white_spaces_toggled)
+    def on_spinbutton_embfile_size_value_changed(spinbutton):
+        dad.embfile_size = int(spinbutton_embfile_size.get_value())
+        if not dad.embfile_size_mod:
+            dad.embfile_size_mod = True
+            support.dialog_info_after_restart(pref_dialog)
+    spinbutton_embfile_size.connect('value-changed', on_spinbutton_embfile_size_value_changed)
     def on_spinbutton_limit_undoable_steps_value_changed(spinbutton):
         dad.limit_undoable_steps = int(spinbutton.get_value())
     spinbutton_limit_undoable_steps.connect('value-changed', on_spinbutton_limit_undoable_steps_value_changed)
@@ -703,7 +797,7 @@ def preferences_tab_text_nodes(dad, vbox_text_nodes):
         checkbutton_spell_check.set_sensitive(False)
         combobox_spell_check_lang.set_sensitive(False)
 
-def preferences_tab_code_nodes(dad, vbox_code_nodes):
+def preferences_tab_plain_text_n_code_nodes(dad, vbox_code_nodes, pref_dialog):
     """Preferences Dialog, Code Nodes Tab"""
     for child in vbox_code_nodes.get_children(): child.destroy()
 
@@ -727,7 +821,7 @@ def preferences_tab_code_nodes(dad, vbox_code_nodes):
     vbox_syntax.pack_start(checkbutton_show_white_spaces, expand=False)
     vbox_syntax.pack_start(checkbutton_highlight_current_line, expand=False)
 
-    frame_syntax = gtk.Frame(label="<b>"+_("Automatic Syntax Highlighting")+"</b>")
+    frame_syntax = gtk.Frame(label="<b>"+_("Code Nodes")+"</b>")
     frame_syntax.get_label_widget().set_use_markup(True)
     frame_syntax.set_shadow_type(gtk.SHADOW_NONE)
     align_syntax = gtk.Alignment()
@@ -741,18 +835,20 @@ def preferences_tab_code_nodes(dad, vbox_code_nodes):
         new_style = dad.style_scheme_liststore[new_iter][0]
         if new_style != dad.style_scheme:
             dad.style_scheme = new_style
-            support.dialog_info_after_restart(dad.window)
+            support.dialog_info_after_restart(pref_dialog)
     combobox_style_scheme.connect('changed', on_combobox_style_scheme_changed)
     def on_checkbutton_show_white_spaces_toggled(checkbutton):
         dad.show_white_spaces = checkbutton.get_active()
-        support.dialog_info_after_restart(dad.window)
+        if dad.syntax_highlighting != cons.RICH_TEXT_ID:
+            dad.sourceview.set_draw_spaces(codeboxes.DRAW_SPACES_FLAGS if dad.show_white_spaces else 0)
     checkbutton_show_white_spaces.connect('toggled', on_checkbutton_show_white_spaces_toggled)
     def on_checkbutton_highlight_current_line_toggled(checkbutton):
         dad.highl_curr_line = checkbutton.get_active()
-        support.dialog_info_after_restart(dad.window)
+        if dad.syntax_highlighting != cons.RICH_TEXT_ID:
+            dad.sourceview.set_highlight_current_line(dad.highl_curr_line)
     checkbutton_highlight_current_line.connect('toggled', on_checkbutton_highlight_current_line_toggled)
 
-def preferences_tab_tree(dad, vbox_tree):
+def preferences_tab_tree(dad, vbox_tree, pref_dialog):
     """Preferences Dialog, Tree Tab"""
     for child in vbox_tree.get_children(): child.destroy()
 
@@ -944,7 +1040,7 @@ def preferences_tab_tree(dad, vbox_tree):
             dad.hpaned.set_property('position', tree_width)
     checkbutton_tree_right_side.connect('toggled', on_checkbutton_tree_right_side_toggled)
 
-def preferences_tab_fonts(dad, vbox_fonts):
+def preferences_tab_fonts(dad, vbox_fonts, pref_dialog):
     """Preferences Dialog, Fonts Tab"""
     for child in vbox_fonts.get_children(): child.destroy()
 
@@ -984,12 +1080,12 @@ def preferences_tab_fonts(dad, vbox_fonts):
     vbox_fonts.pack_start(frame_fonts, expand=False)
     def on_fontbutton_text_font_set(picker):
         dad.text_font = picker.get_font_name()
-        if dad.curr_tree_iter and dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if dad.curr_tree_iter and dad.syntax_highlighting == cons.RICH_TEXT_ID:
             dad.sourceview.modify_font(pango.FontDescription(dad.text_font))
     fontbutton_text.connect('font-set', on_fontbutton_text_font_set)
     def on_fontbutton_code_font_set(picker):
         dad.code_font = picker.get_font_name()
-        if dad.curr_tree_iter and dad.syntax_highlighting != cons.CUSTOM_COLORS_ID:
+        if dad.curr_tree_iter and dad.syntax_highlighting != cons.RICH_TEXT_ID:
             dad.sourceview.modify_font(pango.FontDescription(dad.code_font))
     fontbutton_code.connect('font-set', on_fontbutton_code_font_set)
     def on_fontbutton_tree_font_set(picker):
@@ -997,7 +1093,7 @@ def preferences_tab_fonts(dad, vbox_fonts):
         dad.set_treeview_font()
     fontbutton_tree.connect('font-set', on_fontbutton_tree_font_set)
 
-def preferences_tab_links(dad, vbox_links):
+def preferences_tab_links(dad, vbox_links, pref_dialog):
     """Preferences Dialog, Links Tab"""
     for child in vbox_links.get_children(): child.destroy()
 
@@ -1124,25 +1220,28 @@ def preferences_tab_links(dad, vbox_links):
     entry_custom_folderlink_cmd.connect('changed', on_entry_custom_folderlink_cmd_changed)
     def on_spinbutton_anchor_size_value_changed(spinbutton):
         dad.anchor_size = int(spinbutton_anchor_size.get_value())
+        if not dad.anchor_size_mod:
+            dad.anchor_size_mod = True
+            support.dialog_info_after_restart(pref_dialog)
     spinbutton_anchor_size.connect('value-changed', on_spinbutton_anchor_size_value_changed)
     def on_colorbutton_col_link_webs_color_set(colorbutton):
         dad.col_link_webs = "#" + colorbutton.get_color().to_string()[1:]
-        support.dialog_info_after_restart(dad.window)
+        support.dialog_info_after_restart(pref_dialog)
     colorbutton_col_link_webs.connect('color-set', on_colorbutton_col_link_webs_color_set)
     def on_colorbutton_col_link_node_color_set(colorbutton):
         dad.col_link_node = "#" + colorbutton.get_color().to_string()[1:]
-        support.dialog_info_after_restart(dad.window)
+        support.dialog_info_after_restart(pref_dialog)
     colorbutton_col_link_node.connect('color-set', on_colorbutton_col_link_node_color_set)
     def on_colorbutton_col_link_file_color_set(colorbutton):
         dad.col_link_file = "#" + colorbutton.get_color().to_string()[1:]
-        support.dialog_info_after_restart(dad.window)
+        support.dialog_info_after_restart(pref_dialog)
     colorbutton_col_link_file.connect('color-set', on_colorbutton_col_link_file_color_set)
     def on_colorbutton_col_link_fold_color_set(colorbutton):
         dad.col_link_fold = "#" + colorbutton.get_color().to_string()[1:]
-        support.dialog_info_after_restart(dad.window)
+        support.dialog_info_after_restart(pref_dialog)
     colorbutton_col_link_fold.connect('color-set', on_colorbutton_col_link_fold_color_set)
 
-def preferences_tab_misc(dad, vbox_misc):
+def preferences_tab_misc(dad, vbox_misc, pref_dialog):
     """Preferences Dialog, Misc Tab"""
     for child in vbox_misc.get_children(): child.destroy()
 
@@ -1180,10 +1279,19 @@ def preferences_tab_misc(dad, vbox_misc):
     hbox_autosave.pack_start(label_autosave, expand=False)
     checkbutton_autosave_on_quit = gtk.CheckButton(_("Autosave on Quit"))
     checkbutton_backup_before_saving = gtk.CheckButton(_("Create a Backup Copy Before Saving"))
+    hbox_num_backups = gtk.HBox()
+    hbox_num_backups.set_spacing(4)
+    label_num_backups = gtk.Label(_("Number of Backups to Keep"))
+    adjustment_num_backups = gtk.Adjustment(value=dad.backup_num, lower=1, upper=100, step_incr=1)
+    spinbutton_num_backups = gtk.SpinButton(adjustment_num_backups)
+    spinbutton_num_backups.set_sensitive(dad.backup_copy)
+    hbox_num_backups.pack_start(label_num_backups, expand=False)
+    hbox_num_backups.pack_start(spinbutton_num_backups, expand=False)
     vbox_saving.pack_start(hbox_autosave, expand=False)
     vbox_saving.pack_start(checkbutton_autosave_on_quit, expand=False)
     vbox_saving.pack_start(checkbutton_backup_before_saving, expand=False)
-
+    vbox_saving.pack_start(hbox_num_backups, expand=False)
+    
     checkbutton_autosave.set_active(dad.autosave[0])
     spinbutton_autosave.set_value(dad.autosave[1])
     spinbutton_autosave.set_sensitive(dad.autosave[0])
@@ -1294,7 +1402,11 @@ def preferences_tab_misc(dad, vbox_misc):
     spinbutton_autosave.connect('value-changed', on_spinbutton_autosave_value_changed)
     def on_checkbutton_backup_before_saving_toggled(checkbutton):
         dad.backup_copy = checkbutton.get_active()
+        spinbutton_num_backups.set_sensitive(dad.backup_copy)
     checkbutton_backup_before_saving.connect('toggled', on_checkbutton_backup_before_saving_toggled)
+    def on_spinbutton_num_backups_value_changed(spinbutton):
+        dad.backup_num = int(spinbutton.get_value())
+    spinbutton_num_backups.connect('value-changed', on_spinbutton_num_backups_value_changed)
     def on_checkbutton_autosave_on_quit_toggled(checkbutton):
         dad.autosave_on_quit = checkbutton.get_active()
     checkbutton_autosave_on_quit.connect('toggled', on_checkbutton_autosave_on_quit_toggled)
@@ -1339,8 +1451,8 @@ def dialog_preferences(dad):
     notebook = gtk.Notebook()
     notebook.set_tab_pos(gtk.POS_LEFT)
     notebook.append_page(tabs_vbox_vec[0], gtk.Label(_("All Nodes")))
-    notebook.append_page(tabs_vbox_vec[1], gtk.Label(_("Text Nodes")))
-    notebook.append_page(tabs_vbox_vec[2], gtk.Label(_("Code Nodes")))
+    notebook.append_page(tabs_vbox_vec[1], gtk.Label(_("Rich Text")))
+    notebook.append_page(tabs_vbox_vec[2], gtk.Label(_("Plain Text and Code")))
     notebook.append_page(tabs_vbox_vec[3], gtk.Label(_("Tree")))
     notebook.append_page(tabs_vbox_vec[4], gtk.Label(_("Fonts")))
     notebook.append_page(tabs_vbox_vec[5], gtk.Label(_("Links")))
@@ -1348,8 +1460,8 @@ def dialog_preferences(dad):
 
     tab_constructor = {
         0: preferences_tab_all_nodes,
-        1: preferences_tab_text_nodes,
-        2: preferences_tab_code_nodes,
+        1: preferences_tab_rich_text_nodes,
+        2: preferences_tab_plain_text_n_code_nodes,
         3: preferences_tab_tree,
         4: preferences_tab_fonts,
         5: preferences_tab_links,
@@ -1358,7 +1470,7 @@ def dialog_preferences(dad):
 
     def on_notebook_switch_page(notebook, page, page_num):
         #print "new page", page_num
-        tab_constructor[page_num](dad, tabs_vbox_vec[page_num])
+        tab_constructor[page_num](dad, tabs_vbox_vec[page_num], dialog)
         tabs_vbox_vec[page_num].show_all()
     notebook.connect('switch-page', on_notebook_switch_page)
 
